require "satoshi_ruby/version"
require 'httparty'
require 'queryparams'
require 'bigdecimal'
require 'satoshi_ruby/utilities/currency_utility'

module SatoshiRuby

  class Satoshi
  	include HTTParty

  	URL = 'https://api.vaultofsatoshi.com'

  	def initialize api_key, api_sign
  		@api_key = api_key
  		@api_sign = api_sign
  	end

  	def wallet_address cur = ''
  		end_point = '/info/wallet_address'
  		data = {currency: cur.to_s}
  		action end_point, data
  	end

  	def currency cur = ''
  		end_point = '/info/currency'
  		data = {currency: cur.to_s}
  		action end_point, data
  	end

  	def account
  		end_point = '/info/account'
  		action end_point, nil
  	end

  	def balance cur = ''
  		end_point = '/info/balance'
  		data = {currency: cur.to_s}
  		action end_point, data
  	end

  	def wallet_history cur, count = 100, after = ''
  		end_point = '/info/wallet_history'
  		data = {currency: cur.to_s, count: count, after: after}
  		action end_point, data
  	end

  	def ticker order_currency, payment_currency
  		end_point = '/info/ticker'
  		data = {order_currency: order_currency, payment_currency: payment_currency}
  		action end_point, data
  	end

  	def quote type, order_currency, units, payment_currency, price
  		end_point = '/info/quote'
  		data = {type: type, order_currency: order_currency, units: units, payment_currency: payment_currency,
  			price: price}
  		action end_point, data
  	end

  	def orderbook order_currency, payment_currency, group_orders, count
  		end_point = '/info/orderbook'
  		data = {order_currency: order_currency, payment_currency: payment_currency, group_orders: group_orders,
  			count: count}
  		action end_point, data
  	end

  	def orders count = 100, after = '', open_only = ''
  		end_point = '/info/orders'
  		data = {count: count, after: after}
  		data = open_only.empty? ? data : data.merge({open_only: open_only})
  		action end_point, data
  	end

  	def order_details order_id
  		end_point = '/info/order_detail'
  		data = {order_id: order_id}
  		action end_point, data
  	end

  	def place type, order_currency, units, payment_currency, price
  		end_point = '/trade/place'
  		data = {type: type, order_currency: order_currency, units: units, payment_currency: payment_currency,
  			price: price}
  		action end_point, data
  	end

  	def cancel order_id
  		end_point = '/trade/cancel'
  		data = {order_id: order_id}
  		action end_point, data
  	end

  	private

  	def get_nonce
  		time = Time.now
  		time.to_i.to_s + time.usec.to_s
  	end

  	def sign base_string
  	  hmac = OpenSSL::HMAC.hexdigest('sha512', @api_sign, base_string)
  	  Base64.encode64(hmac).gsub("\n", '')
  	end

  	def path end_point
  		URL + end_point
  	end

  	def action end_point, data = ''
  		nonce = get_nonce
  		data = data.nil? ? {nonce: nonce} : data.merge({nonce: nonce})
  		signed = sign(end_point + 0.chr + parameterize(data))
  		options = {body: data, headers: {'Api-Key' => @api_key, 'Api-Sign' => signed}}
  		res = self.class.post(path(end_point), options)
  		puts res
  		res
  	end

  	def parameterize(params)
  	  URI.escape(QueryParams.encode(params))
  	end

  end
end
